'''A concrete apport.PackageInfo class implementation for openSUSE.

Copyright (C) 2008 Nikolay Derkach
Author: Nikolay Derkach <nderkach@gmail.com>

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
the full text of the license.
'''

import os
import rpm
from packaging_rpm import RPMPackageInfo

# FIXME: temporarily
import sys
sys.path.insert(0, '/home/nderkach/development/osdebug/zypp/trunk/libzypp-bindings/swig/python')
#

import zypp

class __SUSEPackageInfo(RPMPackageInfo):
    '''Concrete apport.PackageInfo class implementation for openSUSE.'''
    
    # some helper functions from rpmUtils.miscutils (yum)
    
    def compareEVR(self, (e1, v1, r1), (e2, v2, r2)):
        # return 1: a is newer than b
        # 0: a and b are the same version
        # -1: b is newer than a
        e1 = str(e1)
        v1 = str(v1)
        r1 = str(r1)
        e2 = str(e2)
        v2 = str(v2)
        r2 = str(r2)
        #print '%s, %s, %s vs %s, %s, %s' % (e1, v1, r1, e2, v2, r2)
        rc = rpm.labelCompare((e1, v1, r1), (e2, v2, r2))
        #print '%s, %s, %s vs %s, %s, %s = %s' % (e1, v1, r1, e2, v2, r2, rc)
        return rc
    
    def stringToVersion(self, verstring):
        if verstring in [None, '']:
            return (None, None, None)
        i = verstring.find(':')
        if i != -1:
            try:
                epoch = str(long(verstring[:i]))
            except ValueError:
                # look, garbage in the epoch field, how fun, kill it
                epoch = '0' # this is our fallback, deal
        else:
            epoch = '0'
        j = verstring.find('-')
        if j != -1:
            if verstring[i + 1:j] == '':
                version = None
            else:
                version = verstring[i + 1:j]
            release = verstring[j + 1:]
        else:
            if verstring[i + 1:] == '':
                version = None
            else:
                version = verstring[i + 1:]
            release = None
        return (epoch, version, release)

    # This information should be delivered by the crash database server:
    # A list of ids of official keys used by the openSUSE (suse-build-key)
    official_keylist = (
        '9c800aca', # SuSE Package Signing Key <build@suse.de>
        '307e3d54', # SuSE Package Signing Key <build@suse.de>
        'a1912208', # Novell Provo Build <novell-provo-build@novell.com>
        '7e2e3b05', # Novell Provo Build <novell-provo-build@novell.com>
        '3d25d3d9', # SuSE Security Team <security@suse.de>
        '0dfb3188', # Open Enterprise Server <support@novell.com>
        '1d061a62', # build@novell.com (Novell Linux Products) <support@novell.com>
        '56b4177a', # openSUSE:Factory OBS Project (external build service)
        '3dbdc284', # openSUSE Project Signing Key <opensuse@opensuse.org>
        )

    # This information should be delivered by the crash database server:
    # A list of official vendor strings used by openSUSE
    official_vendorlist = (
        'SUSE LINUX Products GmbH, Nuernberg, Germany',
        'openSUSE Build Service',
        'openSUSE',
        )

    def is_distro_package(self, package):
        '''Check if a package is a genuine distro package (True) or comes from
        a third-party source.'''
        if self._get_header(package)['vendor'] in self.official_vendorlist:
            if RPMPackageInfo.is_distro_package(self, package):
                # GPG key id checks out OK. Yay!
                return True
        else:
            return False

    def get_source_tree(self, srcpackage, dir, version=None):
        '''Download given source package and unpack it into dir (which should
        be empty).

        This also has to care about applying patches etc., so that dir will
        eventually contain the actually compiled source.

        If version is given, this particular version will be retrieved.
        Otherwise this will fetch the latest available version.

        Return the directory that contains the actual source root directory
        (which might be a subdirectory of dir). Return None if the source is
        not available.'''
        # Used only by apport-retrace.
        # FIXME STUB
        return None
         
    def compare_versions(self, ver1, ver2):
        '''Compare two package versions.

        Return -1 for ver < ver2, 0 for ver1 == ver2, and 1 for ver1 > ver2.'''
        # Used by crashdb.py (i.e. the frontends)
        return self.compareEVR(self.stringToVersion(ver1), self.stringToVersion(ver2))
        
    def get_file_package(self, file):
        '''Return the package a file belongs to, or None if the file is not
        shipped by any package.
        
        Under normal use, the 'file' argument will always be the executable
        that crashed.
        '''  
        
        hdrs = self._get_headers_by_tag('basenames',file)
        h = None
        if len(hdrs) == 1: # If the file belongs to multiple packages
        # there is some --force package installation
        # FIXME: implement some more smart hadling 
            h = hdrs[0]
        return h['n']


    # ZYpp singleton intitialization
    __zypp_instance__ = None

    def get_zypp_instance(self):

        if self.__zypp_instance__ is None:
            self.__zypp_instance__ = zypp.ZYppFactory_instance().getZYpp()
            repoManager = zypp.RepoManager()
            repos = repoManager.knownRepositories()

            for repo in repos:
                if not repo.enabled():
                    continue
                if os.geteuid() == 0 and not repoManager.isCached(repo):
                    repoManager.refreshMetadata(repo)
                    repoManager.buildCache(repo)
                if repoManager.isCached(repo):
                    repoManager.loadFromCache(repo)

        return self.__zypp_instance__

    def get_available_version(self, package):
        '''Return the latest available version of a package.'''

        myArch = RPMPackageInfo.get_architecture(self, package)
        myZ = self.get_zypp_instance()
        version = None
        
        for item in myZ.pool():
            if zypp.isKindPackage(item) and item.name() == package and str(item.arch()) == myArch:
                if (version == None) or (self.compare_versions(version, str(item.edition())) > 0):
                    version = str(item.edition())

        return version

impl = __SUSEPackageInfo()

#
# Unit test
#

if __name__ == '__main__':
    import unittest

    class __SUSEPackageInfoTest(unittest.TestCase):

        def test_is_distro_package(self):
            '''Test is_distro_package().'''

            self.assert_(impl.is_distro_package('kernel-default'))
            try:
                self.assert_(not impl.is_distro_package('libxine1'))
            except ValueError:
                pass
            self.assertRaises(ValueError, impl.is_distro_package, 'nonexistant_package') 
            
        def test_compare_versions(self):
            '''Test is_distro_package().'''
            
            self.assertEqual(impl.compare_versions('3.2-112.1', '3.2-113.2'), -1)
            
        def test_get_file_package(self):
            '''Test get_file_package().'''
            
            package = impl.get_file_package('/bin/bash') 
            self.assertEqual(package, 'bash')

        def test_get_header(self):
            '''Test _get_header().'''
            
            hdr = impl._get_header('bash-3.2-112.x86_64')
            self.assertEqual(hdr['n'], 'bash')
            hdr = impl._get_header('bash-3.2')
            self.assertEqual(hdr['n'], 'bash')      
            
        def test_get_available_version(self):
            '''Test get_available_version().'''
            
            self.assert_(impl.get_available_version('bash'))

    unittest.main()
